/*
 *  Pedometer - Android App
 *  Copyright (C) 2009 Levente Bagi
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * This version of StepDetector.java was edited by Shelby Kirn no
 * later than Sunday, April 30, 2017.
 */

package com.example.shelby.healthquest40;

import android.content.Context;
import android.content.SharedPreferences;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.preference.PreferenceManager;
import android.util.Log;

import java.util.ArrayList;

/**
 * Detects steps and notifies all listeners (that implement StepListener).
 * @author Levente Bagi
 */

public class StepDetector implements SensorEventListener
{
    private final static String TAG = "StepDetector"; //for registering steps in logcat
    private float   mLimit = 10; //threshold for a step
    private float   mLastValues[] = new float[3*2]; //holds the most recent values
    private float   mScale[] = new float[2]; //accounts for gravity
    private float   mYOffset;

    private float   mLastDirections[] = new float[3*2]; //holds the most recent direction
    private float   mLastExtremes[][] = { new float[3*2], new float[3*2] }; //used to calculate the difference
    private float   mLastDiff[] = new float[3*2]; //the last difference
    private int     mLastMatch = -1; //used for the sensor changes

    public ArrayList<StepListener> mStepListeners = new ArrayList<StepListener>();

    public StepDetector() {
        /************************************************************************
         * Initializes the differences in the variables that qualify as a step. *
         ************************************************************************/
        int h = 480;
        mYOffset = h * 0.5f;
        mScale[0] = - (h * 0.5f * (1.0f / (SensorManager.STANDARD_GRAVITY * 2)));
        mScale[1] = - (h * 0.5f * (1.0f / (SensorManager.MAGNETIC_FIELD_EARTH_MAX)));
    }

    public void addStepListener(StepListener sl) {
        /***********************************************
         * Adds a Step Listener. Called by other fcns. *
         ***********************************************/
        mStepListeners.add(sl);
    }


    public void onSensorChanged(SensorEvent event) {
        /****************************************************************
         * This function is what actually registers the sensor change.  *
         * It checks to make sure that it registered the accelerometer  *
         * before running the changed values though a series of checks. *
         * If the values pass all the checks, then it is registered as  *
         * a step, and the step value is incremented by one.            *
         ****************************************************************/
        Sensor sensor = event.sensor;
        synchronized (this) {
            if (sensor.getType() == Sensor.TYPE_ORIENTATION) {
            } //ignore the sensor change if it is the orientation
            else {
                int j = (sensor.getType() == Sensor.TYPE_ACCELEROMETER) ? 1 : 0;
                if (j == 1) { //if it is the accelerometer, do not ignore it
                    float vSum = 0;
                    for (int i=0 ; i<3 ; i++) {
                        final float v = mYOffset + event.values[i] * mScale[j];
                        vSum += v;
                    }
                    int k = 0;
                    float v = vSum / 3;

                    float direction = (v > mLastValues[k] ? 1 : (v < mLastValues[k] ? -1 : 0));
                    if (direction == - mLastDirections[k]) {
                        // Direction changed
                        int extType = (direction > 0 ? 0 : 1); // minimum or maximum?
                        mLastExtremes[extType][k] = mLastValues[k];
                        float diff = Math.abs(mLastExtremes[extType][k] - mLastExtremes[1 - extType][k]);

                        if (diff > mLimit) {

                            boolean isAlmostAsLargeAsPrevious = diff > (mLastDiff[k]*2/3);
                            boolean isPreviousLargeEnough = mLastDiff[k] > (diff/3);
                            boolean isNotContra = (mLastMatch != 1 - extType);


                            if (isAlmostAsLargeAsPrevious && isPreviousLargeEnough && isNotContra) {
                                /**********************************************************************
                                 * this marks steps, but sometimes registers steps even if the phone  *
                                 * is stationary. when it does, it registers a bunch within a second- *
                                 * usually about ten milliseconds apart.                              *
                                 **********************************************************************/
                                /**********************************************************************
                                 * TODO: find a way to throttle this. if it registers more than four  *
                                 * TODO: steps per second, stop it from counting for the rest of the  *
                                 * TODO: second.                                                      *
                                 **********************************************************************/

                                Log.i(TAG, "step"); //this pings in the logcat every time a step is counted

                                /********************************
                                 * setting up sharedpreferences *
                                 ********************************/
                                Context context = MainActivity.getContextofApplication();
                                SharedPreferences myPreferences = PreferenceManager.getDefaultSharedPreferences(context);
                                SharedPreferences.Editor myEditor = myPreferences.edit();

                                /***********************************
                                 * increment the step count by one *
                                 ***********************************/
                                myEditor.putFloat("DAILY_STEPS", (myPreferences.getFloat("DAILY_STEPS", 0)+1));
                                myEditor.commit();

                                for (StepListener stepListener : mStepListeners) {
                                    stepListener.onStep(); //notifies distance
                                }
                                mLastMatch = extType; //sets up for the next sensor change
                            }
                            else {
                                mLastMatch = -1;
                            }
                        }
                        mLastDiff[k] = diff; //resets the last difference
                    }
                    mLastDirections[k] = direction;
                    mLastValues[k] = v;
                }
            }
        }
    }

    public void onAccuracyChanged(Sensor sensor, int accuracy) {
        // don't do anything
    }

}